////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// Julia.cpp
// Julia set calculator class implementation
//
////////////////////////////////////////////////////////////////////////////////

#include <string>
#include <sstream>

#include "Julia.h"


////////////////////////////////////////////////////////////////////////////////

Julia::Julia() :
   k_re (-0.565072),
   k_im ( 0.491657)
   {
   type_name = JULIA_TYPE_NAME;
   re_min = -2.0;
   im_max = 2.0;
   size = 4.0;
   max_iter = 50;
   calc_needed = true;
   set_display_name();
   }


////////////////////////////////////////////////////////////////////////////////

bool Julia::calc_set()
   {
   if (!surface) return false;

   // Code from http://warp.povusers.org/Julia/

   if (calc_needed)
      {
      double re_scale = (size)/(surface->w-1);
      double im_scale = (size)/(surface->h-1);

      int r, g, b;

      Uint32 black = SDL_MapRGB(surface->format, 0, 0, 0);
      Uint32 color;
      int halfway = max_iter/2 - 1;
      float step = (float) 255 / halfway;

      unsigned x, y, n;

      for(y = 0; y < surface->h; ++y)
	 {
	 double c_im = im_max - y * im_scale;
	 for(x = 0; x < surface->w; ++x)
	    {
	    double c_re = re_min + x * re_scale;

	    double z_re = c_re, z_im = c_im;
	    bool in_set = true;
	    for(n = 0; n < max_iter; ++n)
	       {
	       double z_re2 = z_re * z_re, z_im2 = z_im * z_im;
	       if(z_re2 + z_im2 > 4)
		  {
		  in_set = false;
		  break;
		  }
	       z_im = 2*z_re * z_im + k_im;
	       z_re = z_re2 - z_im2 + k_re;
	       }
	    if(in_set) 
	       { 
	       draw_pixel(x, y, &black); 
	       }
	    else
	       {
	       if (n <= halfway)
		  {
		  r = n * step;
		  b = g = 0;
		  }
	       else 
		  {
		  r = 255;
		  b = g = (n - halfway) * step;
		  }
	       color = SDL_MapRGB(surface->format, r, g, b);
	       draw_pixel(x, y, &color); 
	       }
	    }
	 }
      }

   calc_needed = false;
   return true;
   }


////////////////////////////////////////////////////////////////////////////////
bool Julia::init(SDL_Surface * surf)
   {
   if (!surf)
      {
      return false;
      }
   else
      {
      surface = surf;
      }
   calc_needed = true;
   return true;
   }


////////////////////////////////////////////////////////////////////////////////
bool Julia::get_option(std::string option, void * value)
   {
   if (option.compare("k") == 0)
      {
      ((double *)value)[0] = k_re;
      ((double *)value)[0] = k_im;
      }
   else
      {
      return false;
      }

   return true;
   }


////////////////////////////////////////////////////////////////////////////////
bool Julia::set_option(std::string option, void * value)
   {
   if (option.compare("k") == 0)
      {
      k_re = ((double *)value)[0];
      k_im = ((double *)value)[1];
      set_display_name();
      }
   else
      {
      return false;
      }


   return true;
   }


////////////////////////////////////////////////////////////////////////////////
const char * Julia::get_type_name()
   {
   return JULIA_TYPE_NAME;
   }


////////////////////////////////////////////////////////////////////////////////

void Julia::draw_pixel(int x, int y, Uint32 * color)
   {
   Fractal::draw_pixel(x, y, color);
   }


////////////////////////////////////////////////////////////////////////////////
void Julia::set_display_name()
   {
   std::stringstream ss;
   ss << "Julia set K = (" << k_re << "," << k_im << ")";
   display_name = ss.str();
   }
